from os.path import join
from ament_index_python.packages import get_package_share_directory
from launch import LaunchDescription
from launch_ros.actions import Node
from launch.actions import ExecuteProcess

def generate_launch_description():
    launch_root = get_package_share_directory('ar3_config')
    description_root = get_package_share_directory('ar3_description')
    config_root = join(launch_root, 'config')

    urdf_path = join(description_root, 'ar3.urdf')
    with open(urdf_path, 'r') as f:
        robot_description = {'robot_description': f.read()}

    return LaunchDescription([
        # RViz
        Node(
            package='rviz2',
            executable='rviz2',
            output='log',
            arguments=['-d', join(launch_root, 'rviz', 'ar3.rviz')],
            parameters=[robot_description]),

        # Attatch robot to map
        Node(
            package='tf2_ros',
            executable='static_transform_publisher',
            output='screen',
            arguments=['0', '0', '0', '0', '0', '0', 'map', 'base_link']),

        # Controller
        Node(
            package='controller_manager',
            executable='ros2_control_node',
            output='screen',
            parameters=[robot_description, join(config_root, 'controller_manager.yaml')]),

        # Start controllers
        ExecuteProcess(
            cmd=['ros2 run controller_manager spawner.py joint_trajectory_controller'],
            shell=True,
            output='screen'),
        ExecuteProcess(
            cmd=['ros2 run controller_manager spawner.py joint_state_controller'],
            shell=True,
            output='screen'),

        # Robot state publisher
        Node(
            package='robot_state_publisher',
            executable='robot_state_publisher',
            output='screen',
            parameters=[robot_description]),

        # Fake robot state publisher
        Node(
            package='ar3_description',
            executable='state_publisher_net',
            name='state_publisher_net',
            emulate_tty=True,
            output='screen'),
    ])
